@@ -41,10 +41,16 @@ plotmo.singles.glmnet <- function(object, x, nresponse, trace, all1)
     nsingles <- sum(abs(beta) > 1e-8) # drop zero coefs
     order[seq_len(nsingles)]
 }
-# Note: you have to manually assign x and y to the cv.glmnet object
-# before calling plotmo, because cv.glmnet objects do not have a call
-# field (checked in glmnet version 2.0.2).
-
+plotmo.prolog.cv.glmnet <- function(object, object.name, trace, ...) # invoked when plotmo starts
+{
+    # cv.glmnet objects don't have their call field in the usual place, so fix that
+    # (tested on glmnet version 2.0-2)
+    if(is.null(object[["call"]])) {
+        object$call <- object$glmnet.fit$call
+        stopifnot(!is.null(object$call), is.call(object$call))
+    }
+    object
+}
 plotmo.predict.cv.glmnet <- function(object, newdata, type, ..., TRACE)
 {
     # newx for predict.glmnet must be a matrix not a dataframe,
