@@ -1,9 +1,9 @@
-# test.ltut.R: test modified version of linmod example in
+# test.ltut.R: test modified version of linmod from
 #              Friedrich Leisch "Creating R Packages: A Tutorial"
 #
-# Contains three version of linmod (grep for "### ")
-#   1. original code from tutorial
-#   2. minimal changes for plotmo
+# Contains three version of linmod (grep for "###" or "source")
+#   1. original code from Leisch tutorial
+#   2. minimal changes for Milborrow tutorial (and for plotmo)
 #   3. production version (includes error checks)
 
 options(warn=2) # treat warnings as errors
@@ -25,7 +25,11 @@ expect.err <- function(object, expected.msg="")
     } else
         stop("did not get expected error ", expected.msg)
 }
-check <- function(fit, ref, check.coef.names=TRUE)
+almost.equal <- function(x, y)
+{
+    length(x) == length(y) && max(abs(x - y)) < 1e-10
+}
+check.lm <- function(fit, ref, check.coef.names=TRUE)
 {
     cat("check ", deparse(substitute(fit)), " vs ",
         deparse(substitute(ref)), "\n", sep="")
@@ -34,138 +38,51 @@ check <- function(fit, ref, check.coef.names=TRUE)
     if(check.coef.names)
         stopifnot(names(coef(fit)) == names(coef(ref)))
 
-    # following commented out because fit$coefficients is 3x1 and ref$coefficients is a vector.
-    # Note that fit$coefficients is 3x1 to allow multiple response models in some cases, becomes 3x2.
+    # Following commented out because fit$coefficients
+    # is an n x 1 matrix but ref$coefficients is a vector.
+    # Note that fit$coefficients is n x 1 (and not a vector) to allow
+    # multiple response models in some cases (it becomes n x nresponse).
+    #$$
     # stopifnot(identical(dim(fit$coefficients), dim(ref$coefficients)))
-    stopifnot(length(fit$coefficients) == length(ref$coefficients))
-    stopifnot(max(abs(fit$coefficients - ref$coefficients)) < 1e-10)
 
-    # following commented out because fit$residuals is nx2 and ref$residuals is a vector
+    stopifnot(length(fit$coefficients) == length(ref$coefficients))
+    stopifnot(almost.equal(fit$coefficients, ref$coefficients))
+
     # stopifnot(identical(dim(fit$residuals), dim(ref$residuals)))
     stopifnot(length(fit$residuals) == length(ref$residuals))
-    stopifnot(max(abs(fit$residuals - ref$residuals)) < 1e-10)
+    stopifnot(almost.equal(fit$residuals, ref$residuals))
 
     # stopifnot(identical(dim(fit$fitted.values), dim(ref$fitted.values)))
     stopifnot(length(fit$fitted.values) == length(ref$fitted.values))
-    stopifnot(max(abs(fit$fitted.values - ref$fitted.values)) < 1e-10)
+    stopifnot(almost.equal(fit$fitted.values, ref$fitted.values))
 
     if(!is.null(fit$vcov) && !is.null(ref$vcov)) {
         stopifnot(identical(dim(fit$vcov), dim(ref$vcov)))
         stopifnot(length(fit$vcov) == length(ref$vcov))
-        stopifnot(max(abs(fit$vcov - ref$vcov)) < 1e-10)
+        stopifnot(almost.equal(fit$vcov, ref$vcov))
     }
-    # stopifnot(max(abs(fit$sigma - ref$sigma)) < 1e-10)
-    stopifnot(max(abs(fit$df - ref$df)) < 1e-10)
+    ref.sigma <- ref$sigma
+    if(is.null(ref.sigma)) # in lm models, sigma is only available from summary()
+        ref.sigma <- summary(ref)$sigma
+    stopifnot(almost.equal(fit$sigma, ref.sigma))
 
-    stopifnot(max(abs(fitted(fit) - fitted(ref))) < 1e-10)
+    stopifnot(almost.equal(fit$df, ref$df))
+
+    stopifnot(almost.equal(fitted(fit), fitted(ref)))
     stopifnot(names(fitted(fit)) == names(fitted(ref)))
 
-    stopifnot(max(abs(residuals(fit) - residuals(ref))) < 1e-10)
+    stopifnot(almost.equal(residuals(fit), residuals(ref)))
     stopifnot(names(residuals(fit)) == names(residuals(ref)))
 
     # TODO this doesn't test predict with newdata
-    stopifnot(max(abs(predict(fit) - predict(ref))) < 1e-10)
+    stopifnot(almost.equal(predict(fit), predict(ref)))
     stopifnot(names(predict(fit)) == names(predict(ref)))
 }
-### original code from tutorial ####################################
+### Version1: original code from tutorial
 
-linmodEst <- function(x, y)
-{
-    ## compute QR-decomposition of x
-    qx <- qr(x)
+source("linmod.leisch.tutorial.R")
 
-    ## compute (x'x)^(-1) x'y
-    coef <- solve.qr(qx, y)
-
-    ## degrees of freedom and standard deviation of residuals
-    df <- nrow(x)-ncol(x)
-    sigma2 <- sum((y - x%*%coef)^2)/df
-
-    ## compute sigma^2 * (x'x)^-1
-    vcov <- sigma2 * chol2inv(qx$qr)
-    colnames(vcov) <- rownames(vcov) <- colnames(x)
-
-    list(coefficients = coef,
-         vcov = vcov,
-         sigma = sqrt(sigma2),
-         df = df)
-}
-print.linmod <- function(x, ...)
-{
-    cat("Call:\n")
-    print(x$call)
-    cat("\nCoefficients:\n")
-    print(x$coefficients)
-}
-summary.linmod <- function(object, ...)
-{
-    se <- sqrt(diag(object$vcov))
-    tval <- coef(object) / se
-
-    TAB <- cbind(Estimate = coef(object),
-                 StdErr = se,
-                 t.value = tval,
-                 p.value = 2*pt(-abs(tval), df=object$df))
-
-    res <- list(call=object$call,
-                coefficients=TAB)
-
-    class(res) <- "summary.linmod"
-    res
-}
-print.summary.linmod <- function(x, ...)
-{
-    cat("Call:\n")
-    print(x$call)
-    cat("\n")
-
-    printCoefmat(x$coefficients, P.value=TRUE, has.Pvalue=TRUE)
-}
-
-linmod <- function(...) { UseMethod("linmod") }
-
-linmod.default <- function(x, y, ...)
-{
-    x <- as.matrix(x)
-    y <- as.numeric(y)
-
-    est <- linmodEst(x, y)
-
-    est$fitted.values <- as.vector(x %*% est$coefficients)
-    est$residuals <- y - est$fitted.values
-    est$call <- match.call()
-
-    class(est) <- "linmod"
-    est
-}
-linmod.formula <- function(formula, data=list(), ...)
-{
-    mf <- model.frame(formula=formula, data=data)
-    x <- model.matrix(attr(mf, "terms"), data=mf)
-    y <- model.response(mf)
-
-    est <- linmod.default(x, y, ...)
-    est$call <- match.call()
-    est$formula <- formula
-    est
-}
-predict.linmod <- function(object, newdata=NULL, ...)
-{
-    if(is.null(newdata))
-        y <- fitted(object)
-    else{
-        if(!is.null(object$formula)){
-            ## model has been fitted using formula interface
-            x <- model.matrix(object$formula, newdata)
-        } else{
-            x <- newdata
-        }
-        y <- as.vector(x %*% coef(object))
-    }
-    y
-}
-
-cat("==functions in the tutorial have issues with predict\n")
+cat("==example issues with predict with functions in the tutorial\n")
 data(trees)
 tr <- trees
 rownames(tr) <- paste("tree", 1:nrow(trees), sep="")
@@ -176,6 +93,7 @@ library(plotmo)
 expect.err(try(plotmo(fit1)), "object 'Volume' not found")
 fit2 <- linmod(cbind(1, tr[,1:2]), tr[,3])
 stopifnot(coef(fit1) == coef(fit2))
+# following fail because newdata is a data.frame not a matrix
 expect.err(try(predict(fit2, newdata=tr[,1:2])), "requires numeric/complex matrix/vector arguments")
 expect.err(try(predict(fit2, newdata=data.frame(Girth=10, Height=80))), "requires numeric/complex matrix/vector arguments")
 expect.err(try(predict(fit2, newdata=as.matrix(data.frame(Girth=10, Height=80)))), "non-conformable arguments")
@@ -194,86 +112,30 @@ plotmo.predict.linmod <- function(object, newdata, ...)
 }
 plotmo(fit1, pt.col=2, caption="fit1 with original tutorial code and plotmo.predict.linmod")
 plotmo(fit2, pt.col=2, caption="fit2 with original tutorial code and plotmo.predict.linmod")
-plotmo.predict.linmod <- NULL
+remove(plotmo.predict.linmod)
 
-### minimal changes version
-## new version of linmod
+### Version2: minimal changes version for vignette "Guidelines for S3 Regression Models"
 
-linmod <- function(...) UseMethod("linmod")
+source("linmod.milbo.tutorial.R")
 
-linmod.fit <- function(x, y) # internal function, not for the casual user
-{                            # first column of x is the intercept (all 1s)
-
-    qx <- qr(x)                             # QR-decomposition of x
-    coef <- solve.qr(qx, y)                 # compute (x'x)^(-1) x'y
-    df.residual <- nrow(x) - ncol(x)        # degrees of freedom
-    sigma2 <- sum((y - x %*% coef)^2) / df.residual  # variance of residuals
-    vcov <- sigma2 * chol2inv(qx$qr)        # covar mat is sigma^2 * (x'x)^(-1)
-    colnames(vcov) <- rownames(vcov) <- colnames(x)
-    fitted.values <- qr.fitted(qx, y)
-
-    fit <- list(coefficients  = coef,
-                residuals     = y - fitted.values,
-                fitted.values = fitted.values,
-                vcov          = vcov,
-                sigma         = sqrt(sigma2),
-                df.residual   = df.residual)
-
-    class(fit) <- "linmod"
-    fit
-}
-linmod.default <- function(x, y, ...)
-{
-    x <- cbind("(Intercept)"=1, as.matrix(x))
-    # as.matrix rather than as.vector allows multiple response models
-    fit <- linmod.fit(x, as.matrix(y))
-    fit$call <- match.call()
-    fit
-}
-linmod.formula <- function(formula, data=parent.frame(), ...)
-{
-    mf <- model.frame(formula=formula, data=data)
-    terms <- attr(mf, "terms")
-    x <- model.matrix(terms, mf)
-    y <- model.response(mf)
-    fit <- linmod.fit(x, y)
-    fit$terms <- terms
-    fit$call <- match.call()
-    fit
-}
-predict.linmod <- function(object, newdata=NULL, ...)
-{
-    if(is.null(newdata))
-        y <- fitted(object)
-    else {
-        if(is.null(object$terms))              # x,y interface
-            x <- cbind(1, as.matrix(newdata))  # columns must be in same order as orig x
-        else {                                 # formula interface
-            terms <- delete.response(object$terms)
-            x <- model.matrix(terms, model.frame(terms, as.data.frame(newdata)))
-        }
-        y <- as.vector(x %*% coef(object))
-    }
-    y
-}
-cat("==with the new version we are ok\n")
+cat("==check that example issues with functions in the tutorial have gone\n")
 fit1.form <- linmod(Volume~., data=tr)
-cat("==print.default(fit1.form)\n")
-print.default(fit1.form)
+cat("==print(summary(fit1.form))\n")
+print(summary(fit1.form))
 stopifnot(abs(predict(fit1.form, newdata=data.frame(Girth=10, Height=80)) - 16.234045) < 1e-5)
 stopifnot(sum(abs(predict(fit1.form, newdata=as.matrix(tr[1:3,])) - c(4.8376597, 4.5538516, 4.8169813))) < 1e-5)
 
 lm.tr <- lm(Volume~., data=tr)
-check(fit1.form, lm.tr)
+check.lm(fit1.form, lm.tr)
 
 fit1.mat <- linmod(tr[,1:2], tr[,3]) # note no need for intercept term
-cat("==print.default(fit1.mat)\n")
-print.default(fit1.form)
-
+cat("==print(summary(fit1.mat))\n")
+print(summary(fit1.mat))
 stopifnot(abs(predict(fit1.mat, newdata=data.frame(Girth=10, Height=80)) - 16.234045) < 1e-5)
 stopifnot(sum(abs(predict(fit1.mat, newdata=tr[1:3,1:2]) - c(4.8376597, 4.5538516, 4.8169813))) < 1e-5)
 stopifnot(abs(predict(fit1.mat, newdata=as.matrix(data.frame(Girth=10, Height=80))) - 16.234045) < 1e-5)
-check(fit1.mat, lm.tr)
+
+check.lm(fit1.mat, lm.tr)
 
 cat("==example plots\n")
 
@@ -299,25 +161,25 @@ cat("==test model building with different numeric args\n")
 x <- tr[,1:2]
 y <- tr[,3]
 fit2.mat <- linmod(x, y)
-check(fit2.mat, lm.tr)
+check.lm(fit2.mat, lm.tr)
 
 # check consistency with lm
 expect.err(try(linmod(y~x)), "invalid type (list) for variable 'x'")
 expect.err(try(lm(y~x)), "invalid type (list) for variable 'x'")
 
 fit3.mat <- linmod(as.matrix(x), as.matrix(y))
-check(fit3.mat, lm.tr)
+check.lm(fit3.mat, lm.tr)
 
 fit4.form <- linmod(y ~ as.matrix(x))
 lm4 <- linmod(y ~ as.matrix(x))
-check(fit4.form, lm4)
+check.lm(fit4.form, lm4)
 stopifnot(coef(fit4.form)  == coef(lm.tr),
           gsub("as.matrix(x)", "", names(coef(fit4.form)), fixed=TRUE)  == names(coef(lm.tr)))
 
 xm <- as.matrix(x)
 fit5.form <- linmod(y ~ xm)
 lm5 <- linmod(y ~ xm)
-check(fit5.form, lm5)
+check.lm(fit5.form, lm5)
 stopifnot(coef(fit5.form)  == coef(lm.tr),
           gsub("xm", "", names(coef(fit5.form)), fixed=TRUE)  == names(coef(lm.tr)))
 
@@ -327,17 +189,17 @@ y1 <- tr[,3]
 lm1 <- linmod(y1~x1)
 
 fit6.mat <- linmod(x1, y1)
-check(fit6.mat, lm1, check.coef.names=FALSE)
+check.lm(fit6.mat, lm1, check.coef.names=FALSE)
 # production version only:
 # stopifnot(coef(fit6.mat) == coef(lm1),
 #           names(coef(fit6.mat)) == c("(Intercept)", "V1")) # names(coef(lm1) are "(Intercept)" "x1"
 
 fit6.form <- linmod(y1~x1)
-check(fit6.form, lm1)
+check.lm(fit6.form, lm1)
 
 cat("==check integer input (sibsp is an integer) \n")
 
-library(earth)
+library(earth) # for etitanic data
 data(etitanic)
 tit <- etitanic[seq(1, nrow(etitanic), by=60), ] # small set of data for tests (18 cases)
 tit$survived <- tit$survived != 0 # convert to logical
@@ -346,23 +208,29 @@ cat(paste(colnames(tit), "=", sapply(tit, class), sep="", collapse=", "), "\n")
 
 fit7.mat <- linmod(tit$age, tit$sibsp)
 lm7 <- lm.fit(cbind(1, tit$age), tit$sibsp)
-stopifnot(coef(fit7.mat) == coef(lm7))# coef names will differ
+stopifnot(coef(fit7.mat) == coef(lm7)) # coef names will differ
 
 fit7.form <- linmod(sibsp~age, data=tit)
 lm7.form  <- lm(sibsp~age, data=tit)
-check(fit7.form, lm7.form)
+check.lm(fit7.form, lm7.form)
 
 fit8.mat <- linmod(tit$sibsp, tit$age)
 lm8 <- lm.fit(cbind(1, tit$sibsp), tit$age)
-stopifnot(coef(fit8.mat) == coef(lm8))# coef names will differ
+stopifnot(coef(fit8.mat) == coef(lm8)) # coef names will differ
 
 fit8.form <- linmod(age~sibsp, data=tit)
 lm8.form  <- lm(age~sibsp, data=tit)
-check(fit8.form, lm8.form)
+check.lm(fit8.form, lm8.form)
+
+# drop=FALSE so response is a data frame
+fit1a.mat <- linmod(trees[,1:2], trees[, 3, drop=FALSE])
+print(fit1a.mat)
+print(summary(fit1.mat))
+plotres(fit1a.mat) # plot caption shows response name "Volume"
 
 cat("==test model building with different non numeric args\n")
 
-library(earth)
+library(earth) # for etitanic data
 data(etitanic)
 tit <- etitanic[seq(1, nrow(etitanic), by=60), ] # small set of data for tests (18 cases)
 tit$survived <- tit$survived != 0 # convert to logical
@@ -371,7 +239,7 @@ cat(paste(colnames(tit), "=", sapply(tit, class), sep="", collapse=", "), "\n")
 
 lm9 <- lm(survived~., data=tit)
 fit9.form <- linmod(survived~., data=tit)
-check(fit9.form, lm9)
+check.lm(fit9.form, lm9)
 
 options(warn=2) # treat warnings as errors
 expect.err(try(linmod(tit[,c(1,3,4,5,6)], tit[,"survived"])), "NAs introduced by coercion")
@@ -384,29 +252,29 @@ expect.err(try(lm(pclass~., data=tit)), "using type = \"numeric\" with a factor
 expect.err(try(linmod(pclass~., data=tit)), "(converted from warning) NAs introduced by coercion")
 expect.err(try(linmod(tit$pclass, tit$survived)), "(converted from warning) NAs introduced by coercion")
 # # production version
-# expect.err(try(linmod(pclass~., data=tit)), "y is not numeric or logical")
+# expect.err(try(linmod(pclass~., data=tit)), "'y' is not numeric or logical")
 options(warn=1)
 
 lm10 <- lm(pclass~., data=tit) # will give warnings
 fit10.form <- linmod(as.numeric(pclass)~., data=tit)
 stopifnot(coef(fit10.form) == coef(lm10))
 stopifnot(names(coef(fit10.form)) == names(coef(lm10)))
-# check(fit10.form, lm10) # fails because lm10 fitted is all NA
+# check.lm(fit10.form, lm10) # fails because lm10 fitted is all NA
 
 # production version: (minimal version just gives warnings and builds lousy model)
-# expect.err(try(linmod(pclass~., data=tit)), "y is not numeric or logical")
-# expect.err(try(linmod(tit[,-1], tit[,1])), "y is not numeric or logical")
-# expect.err(try(linmod(1:10, paste(1:10))), "y is not numeric or logical")
+# expect.err(try(linmod(pclass~., data=tit)), "'y' is not numeric or logical")
+# expect.err(try(linmod(tit[,-1], tit[,1])), "'y' is not numeric or logical")
+# expect.err(try(linmod(1:10, paste(1:10))), "'y' is not numeric or logical")
 
 fit10a.form <- linmod(survived~pclass, data=tit)
 lm10a <- lm(survived~pclass, data=tit)
-check(fit10a.form, lm10a)
+check.lm(fit10a.form, lm10a)
 
 expect.err(try(linmod(paste(1:10), 1:10)), "requires numeric/complex matrix/vector arguments")
 
 lm11 <- lm(as.numeric(pclass)~., data=tit)
 fit11.form <- linmod(as.numeric(pclass)~., data=tit)
-check(fit10.form, lm11)
+check.lm(fit11.form, lm11)
 
 cat("==data.frame with strings\n")
 
@@ -419,16 +287,15 @@ colnames(df.with.string) <- c("num1", "num2", "string")
 
 fit30.form <- linmod(num1~num2, df.with.string)
 lm30       <- lm(num1~num2, df.with.string)
-check(fit30.form, lm30)
+check.lm(fit30.form, lm30)
 
 fit31.form <- linmod(num1~., df.with.string)
 lm31       <- lm(num1~., df.with.string)
-check(fit31.form, lm31)
+check.lm(fit31.form, lm31)
 
-# minimal version
 expect.err(try(linmod(string~., df.with.string)), "non-numeric argument to binary operator")
 # production version
-# expect.err(try(linmod(string~., df.with.string)), "y is not numeric or logical")
+# expect.err(try(linmod(string~., df.with.string)), "'y' is not numeric or logical")
 
 vec <- c(1,2,3,4,3)
 options(warn=2) # treat warnings as errors
@@ -437,7 +304,7 @@ options(warn=1)
 # minimal version
 expect.err(try(linmod(df.with.string, vec)), "NA/NaN/Inf in foreign function call (arg 1)")
 # production version
-# expect.err(try(linmod(df.with.string, vec)), "NA in x")
+# expect.err(try(linmod(df.with.string, vec)), "NA in 'x'")
 
 options(warn=2) # treat warnings as errors
 expect.err(try(linmod(df.with.string, vec)), "NAs introduced by coercion")
@@ -445,7 +312,7 @@ options(warn=1)
 # minimal version
 expect.err(try(linmod(df.with.string, vec)), "NA/NaN/Inf in foreign function call (arg 1)")
 # production version
-# expect.err(try(linmod(df.with.string, vec)), "NA in x")
+# expect.err(try(linmod(df.with.string, vec)), "NA in 'x'")
 
 cat("==more variables  than cases\n")
 
@@ -462,129 +329,82 @@ y3 <- c(1,2,9,4,5)
 # TODO following gives a lousy error message (should be "columns of x are collinear")
 expect.err(try(linmod(y3~x3)), "singular matrix 'a' in 'solve'")
 
-cat("==multiple response model\n")
-# TODO multiple response models not fully tested
+cat("==nrow(x) does not match length(y)\n")
+# note that the production version gives better error messages
 
-y.2 <- cbind(c(1,2,3,4,9), vec^2)
-fit100.mat <- linmod(vec, y.2)
-lm100 <- lm.fit(cbind(1, vec), y.2)
-stopifnot(coef(fit100.mat) == coef(lm100))
-# stopifnot(names(coef(fit100.mat)) == names(coef(lm100)))
+x4 <- matrix(1:10, ncol=2)
+y4 <- c(1,2,9,4)
+expect.err(try(linmod(x4, y4)), "singular matrix 'a' in 'solve'")
 
-### production version of linmod ###################################
+x5 <- matrix(1:10, ncol=2)
+y5 <- c(1,2,9,4,5,9)
+expect.err(try(linmod(x5, y5)), "singular matrix 'a' in 'solve'")
 
-linmod <- function(...) UseMethod("linmod")
+cat("==y has multiple columns\n")
 
-linmod.fit <- function(x, y) # internal function, not for the casual user
-{                            # first column of x is the intercept
-    if(anyNA(x))
-        stop("NA in x")
-    if(!is.numeric(x[[1]]) && !is.logical(x[[1]]))
-        stop("non-numeric column in x")
-    if(anyNA(y))
-        stop("NA in y")
-    if(!is.numeric(y[[1]]) && !is.logical(y[[1]]))
-        stop("y is not numeric or logical")
+vec <- c(1,2,3,4,3)
+y2 <- cbind(c(1,2,3,4,9), vec^2)
+expect.err(try(linmod(vec, y2)), "'qr' and 'y' must have the same number of rows")
+# following does not issue any error message, it should
+# expect.err(try(linmod(y2~vec)), "error message")
 
-    # ensure all columns in x are named (needed for names in vcov etc.)
-    missing.colnames <-
-        if(is.null(colnames(x))) 1:NCOL(x)
-        else                     nchar(colnames(x)) == 0
-    colnames(x)[missing.colnames] <-
-        c("(Intercept)",
-          paste("V", seq_len(NCOL(x)-1), sep=""))[missing.colnames]
+### Version 3: production version of linmod
 
-    qx <- qr(x)                            # QR-decomposition of x
-    if(qx$rank < ncol(x))
-        stop("x is singular")
-    coef <- solve.qr(qx, y)                # compute (x'x)^(-1) x'y
-    df.residual <- nrow(x) - ncol(x)       # degrees of freedom
-    # TODO consider issuing a warning here if df.residual is 0
-    sigma2 <- sum((y - x %*% coef)^2) / df.residual # stddev of residuals
-    vcov <- sigma2 * chol2inv(qx$qr)       # covar mat is sigma^2 * (x'x)^-1
-    fitted.values <- qr.fitted(qx, y)
+source("linmod.R")
 
-    colnames(vcov) <- rownames(vcov) <- colnames(x)
-    names(fitted.values) <- rownames(x)
-    if(NCOL(coef) > 1)                     # multiple response model
-        colnames(coef) <- colnames(x)
-    else
-        names(coef) <- colnames(x)
-
-    fit <- list(coefficients  = coef,
-                residuals     = y - fitted.values,
-                fitted.values = fitted.values,
-                vcov          = vcov,
-                sigma         = sqrt(sigma2),
-                df.residual   = df.residual)
-
-    class(fit) <- "linmod"
-    fit
-}
-linmod.default <- function(x, y, ...)
-{
-    x <- cbind("(Intercept)"=1, as.matrix(x))
-    fit <- linmod.fit(x, as.matrix(y)) # as.matrix rather than as.vector allows multiple response models
-    fit$call <- match.call()
-    fit
-}
-linmod.formula <- function(formula, data=parent.frame(), ...)
-{
-    mf <- model.frame(formula=formula, data=data)
-    terms <- attr(mf, "terms")
-    x <- model.matrix(terms, mf)
-    y <- model.response(mf)
-
-    fit <- linmod.fit(x, y)
-    fit$terms <- terms
-    fit$call <- match.call()
-    fit
-}
-predict.linmod <- function(object, newdata=NULL, ...)
-{
-    if(is.null(newdata))
-        y <- fitted(object)
-    else{
-        if(is.null(object$terms)) {                 # x,y interface
-            x <- cbind(1, as.matrix(newdata))       # columns must be in the same order as orig x
-        } else {                                    # formula interface
-            terms <- delete.response(object$terms)
-            # Note that the following code can issue quite obscure
-            # error messages for bad newdata.  For example
-            #     predict(obj, newdata=1:3)
-            # causes
-            #     eval(expr, envir, enclos) : object 'varname' not found
-            x <- model.matrix(terms, model.frame(terms, as.data.frame(newdata)))
-        }
-        # The following tests suffice to catch all incorrect input (I believe),
-        # but aren't ideal in that they don't always lead you to the cause of
-        # the problem.  For example, strings in newdata that get converted to
-        # factors by model.matrix can cause the wrong number of columns in x.
-        if(ncol(x) != length(object$coefficients))
-            stop("ncol(x) is ", ncol(x), " but should be ",
-                 length(object$coefficients)-1) # -1 for intercept
-        if(anyNA(x))
-            stop("NA in x")
-        if(!is.numeric(x[[1]]) && !is.logical(x[[1]]))
-            stop("non-numeric column in x")
-        y <- as.vector(x %*% coef(object))
-    }
-    y
-}
-cat("==with the production version we are ok\n")
+cat("==check that example issues with functions in the tutorial have gone\n")
 fit1.form <- linmod(Volume~., data=tr)
+cat("==print(summary(fit1.form))\n")
+print(summary(fit1.form))
+lm.tr <- lm(Volume~., data=tr)
+check.lm(fit1.form, lm.tr)
 stopifnot(abs(predict(fit1.form, newdata=data.frame(Girth=10, Height=80)) - 16.234045) < 1e-5)
 stopifnot(sum(abs(predict(fit1.form, newdata=as.matrix(tr[1:3,])) - c(4.8376597, 4.5538516, 4.8169813))) < 1e-5)
 
-lm.tr <- lm(Volume~., data=tr)
-check(fit1.form, lm.tr)
-
 fit1.mat <- linmod(tr[,1:2], tr[,3]) # note no need for intercept term
-
+cat("==print(summary(fit1.mat))\n")
+print(summary(fit1.mat))
+check.lm(fit1.mat, lm.tr)
 stopifnot(abs(predict(fit1.mat, newdata=data.frame(Girth=10, Height=80)) - 16.234045) < 1e-5)
 stopifnot(sum(abs(predict(fit1.mat, newdata=tr[1:3,1:2]) - c(4.8376597, 4.5538516, 4.8169813))) < 1e-5)
 stopifnot(abs(predict(fit1.mat, newdata=as.matrix(data.frame(Girth=10, Height=80))) - 16.234045) < 1e-5)
-check(fit1.mat, lm.tr)
+
+cat("==print.default(fit1.form)\n")
+print.default(fit1.form)
+
+cat("==check single x variable\n")
+fit1a.form <- linmod(Volume~Height, data=tr)
+cat("==print(summary(fit1a.form))\n")
+print(summary(fit1a.form))
+lma.tr <- lm(Volume~Height, data=tr)
+check.lm(fit1a.form, lma.tr)
+
+stopifnot(abs(predict(fit1a.form, newdata=data.frame(Height=80)) - 36.34437) < 1e-5)
+stopifnot(abs(predict(fit1a.form, newdata=data.frame(Girth=99, Height=80)) - 36.34437) < 1e-5)
+stopifnot(sum(abs(predict(fit1a.form, newdata=as.matrix(tr[1:3,])) - c(20.91087, 13.19412, 10.10742))) < 1e-5)
+
+fit1a.mat <- linmod(tr[,2,drop=FALSE], tr[,3])
+cat("==print(summary(fit1a.mat))\n")
+print(summary(fit1a.mat))
+check.lm(fit1a.mat, lma.tr)
+stopifnot(abs(predict(fit1a.mat, newdata=data.frame(Height=80)) - 36.34437) < 1e-5)
+stopifnot(sum(abs(predict(fit1a.mat, newdata=tr[1:3,2]) - c(20.91087, 13.19412, 10.10742))) < 1e-5)
+stopifnot(abs(predict(fit1a.mat, newdata=as.matrix(data.frame(Height=80))) - 36.34437) < 1e-5)
+
+# check that rownames got propagated
+stopifnot(names(fit1.form$residuals)[1] == "tree1")
+stopifnot(names(fit1.form$fitted.values)[3] == "tree3")
+stopifnot(names(fit1.mat$residuals)[1] == "tree1")
+stopifnot(names(fit1.mat$fitted.values)[3] == "tree3")
+stopifnot(!is.null(names(fit1.mat$residuals)))
+stopifnot(!is.null(names(fit1.mat$fitted.values)))
+cat("==print.default(fit1.mat)\n")
+print.default(fit1.mat)
+
+# check that we don't artificially add rownames when no original rownames
+fit1a.mat <- linmod(trees[,1:2], trees[,3])
+stopifnot(is.null(names(fit1a.mat$residuals)))
+stopifnot(is.null(names(fit1a.mat$fitted.values)))
 
 cat("==example plots\n")
 
@@ -605,50 +425,62 @@ plotmo(fit1.mat)
 plotres(fit1.form)
 plotres(fit1.mat)
 
+# test that keep arg works correctly for plotmo and plotres
+fit1.mat.keep <- linmod(trees[,1:2], trees[,3], keep=TRUE)
+fit1.mat.keep$call <- NULL # trick to force use of x and y in plotmo
+plotmo(fit1.mat.keep, pt.col=3)
+plotres(fit1.mat.keep)
+
 cat("==test model building with different numeric args\n")
 
 x <- tr[,1:2]
 y <- tr[,3]
 fit2.mat <- linmod(x, y)
-check(fit2.mat, lm.tr)
+check.lm(fit2.mat, lm.tr)
 
 # check consistency with lm
 expect.err(try(linmod(y~x)), "invalid type (list) for variable 'x'")
 expect.err(try(lm(y~x)), "invalid type (list) for variable 'x'")
 
 fit3.mat <- linmod(as.matrix(x), as.matrix(y))
-check(fit3.mat, lm.tr)
+check.lm(fit3.mat, lm.tr)
 
 fit4.form <- linmod(y ~ as.matrix(x))
 lm4 <- linmod(y ~ as.matrix(x))
-check(fit4.form, lm4)
+check.lm(fit4.form, lm4)
 stopifnot(coef(fit4.form)  == coef(lm.tr),
           gsub("as.matrix(x)", "", names(coef(fit4.form)), fixed=TRUE)  == names(coef(lm.tr)))
 
 xm <- as.matrix(x)
 fit5.form <- linmod(y ~ xm)
 lm5 <- linmod(y ~ xm)
-check(fit5.form, lm5)
+check.lm(fit5.form, lm5)
 stopifnot(coef(fit5.form)  == coef(lm.tr),
           gsub("xm", "", names(coef(fit5.form)), fixed=TRUE)  == names(coef(lm.tr)))
 
+# test that non-intercept models give an error message (for now,
+# until we test them, and remove -1 in predict.linmod)
+
+cat("==test no intercept model gives an error message\n")
+expect.err(try(linmod(Volume~.-1, data=tr)), "the first column of 'x' is not an intercept column (all 1s)")
+
 cat("==test correct use of global x1 and y1\n")
 x1 <- tr[,1]
 y1 <- tr[,3]
 lm1 <- linmod(y1~x1)
 
 fit6.mat <- linmod(x1, y1)
-check(fit6.mat, lm1, check.coef.names=FALSE)
+check.lm(fit6.mat, lm1, check.coef.names=FALSE)
 # production version only:
 stopifnot(coef(fit6.mat) == coef(lm1),
           names(coef(fit6.mat)) == c("(Intercept)", "V1")) # names(coef(lm1) are "(Intercept)" "x1"
 
 fit6.form <- linmod(y1~x1)
-check(fit6.form, lm1)
+check.lm(fit6.form, lm1)
 
 cat("==check integer input (sibsp is an integer) \n")
 
-library(earth)
+library(earth) # for etitanic data
 data(etitanic)
 tit <- etitanic[seq(1, nrow(etitanic), by=60), ] # small set of data for tests (18 cases)
 tit$survived <- tit$survived != 0 # convert to logical
@@ -657,23 +489,29 @@ cat(paste(colnames(tit), "=", sapply(tit, class), sep="", collapse=", "), "\n")
 
 fit7.mat <- linmod(tit$age, tit$sibsp)
 lm7 <- lm.fit(cbind(1, tit$age), tit$sibsp)
-stopifnot(coef(fit7.mat) == coef(lm7))# coef names will differ
+stopifnot(coef(fit7.mat) == coef(lm7)) # coef names will differ
 
 fit7.form <- linmod(sibsp~age, data=tit)
 lm7.form  <- lm(sibsp~age, data=tit)
-check(fit7.form, lm7.form)
+check.lm(fit7.form, lm7.form)
 
 fit8.mat <- linmod(tit$sibsp, tit$age)
 lm8 <- lm.fit(cbind(1, tit$sibsp), tit$age)
-stopifnot(coef(fit8.mat) == coef(lm8))# coef names will differ
+stopifnot(coef(fit8.mat) == coef(lm8)) # coef names will differ
 
 fit8.form <- linmod(age~sibsp, data=tit)
 lm8.form  <- lm(age~sibsp, data=tit)
-check(fit8.form, lm8.form)
+check.lm(fit8.form, lm8.form)
+
+# drop=FALSE so response is a data frame
+fit1a.mat <- linmod(trees[,1:2], trees[, 3, drop=FALSE])
+print(fit1a.mat)
+print(summary(fit1.mat))
+plotres(fit1a.mat) # plot caption shows response name "Volume"
 
 cat("==test model building with different non numeric args\n")
 
-library(earth)
+library(earth) # for etitanic data
 data(etitanic)
 tit <- etitanic[seq(1, nrow(etitanic), by=60), ] # small set of data for tests (18 cases)
 tit$survived <- tit$survived != 0 # convert to logical
@@ -682,10 +520,10 @@ cat(paste(colnames(tit), "=", sapply(tit, class), sep="", collapse=", "), "\n")
 
 lm9 <- lm(survived~., data=tit)
 fit9.form <- linmod(survived~., data=tit)
-check(fit9.form, lm9)
+check.lm(fit9.form, lm9)
 
 expect.err(try(linmod(tit[,c(1,3,4,5,6)], tit[,"survived"])),
-           "non-numeric column in x")
+           "non-numeric column in 'x'")
 fit9a.form <- lm.fit(data.matrix(cbind("(Intercept)"=1, tit[,c(1,3,4,5,6)])), tit[,"survived"])
 lm9 <- lm.fit(data.matrix(cbind("(Intercept)"=1, tit[,c(1,3,4,5,6)])), tit[,"survived"])
 stopifnot(coef(fit9a.form) == coef(lm9))
@@ -693,30 +531,42 @@ stopifnot(names(coef(fit9a.form)) == names(coef(lm9)))
 
 options(warn=2) # treat warnings as errors
 expect.err(try(lm(pclass~., data=tit)), "using type = \"numeric\" with a factor response will be ignored")
-expect.err(try(linmod(pclass~., data=tit)), "y is not numeric or logical")
+expect.err(try(linmod(pclass~., data=tit)), "'y' is not numeric or logical")
 options(warn=1)
 
 lm10 <- lm(pclass~., data=tit) # will give warnings
 fit10.form <- linmod(as.numeric(pclass)~., data=tit)
 stopifnot(coef(fit10.form) == coef(lm10))
 stopifnot(names(coef(fit10.form)) == names(coef(lm10)))
-# check(fit10.form, lm10) # fails because lm10 fitted is all NA
+# check.lm(fit10.form, lm10) # fails because lm10 fitted is all NA
 
-expect.err(try(linmod(pclass~., data=tit)), "y is not numeric or logical")
-expect.err(try(linmod(tit[,-1], tit[,1])), "non-numeric column in x")
-expect.err(try(linmod(1:10, paste(1:10))), "y is not numeric or logical")
+expect.err(try(linmod(pclass~., data=tit)), "'y' is not numeric or logical")
+expect.err(try(linmod(tit[,-1], tit[,1])), "non-numeric column in 'x'")
+expect.err(try(linmod(1:10, paste(1:10))), "'y' is not numeric or logical")
 
 fit10a.form <- linmod(survived~pclass, data=tit)
 lm10a <- lm(survived~pclass, data=tit)
-check(fit10a.form, lm10a)
+check.lm(fit10a.form, lm10a)
 
-expect.err(try(linmod(tit[,"pclass"], tit[,"age"])), "non-numeric column in x")
+expect.err(try(linmod(tit[,"pclass"], tit[,"age"])), "non-numeric column in 'x'")
 
-expect.err(try(linmod(paste(1:10), 1:10)), "non-numeric column in x")
+expect.err(try(linmod(paste(1:10), 1:10)), "non-numeric column in 'x'")
 
 lm11 <- lm(as.numeric(pclass)~., data=tit)
 fit11.form <- linmod(as.numeric(pclass)~., data=tit)
-check(fit10.form, lm11)
+check.lm(fit11.form, lm11)
+
+# logical data (not numeric)
+bool.data <- data.frame(x=rep(c(TRUE, FALSE, TRUE), length.out=10),
+                        y=rep(c(TRUE, FALSE, FALSE), length.out=10))
+lm12 <- lm(y~x, data=bool.data)
+fit12.form <- linmod(y~x, data=bool.data)
+check.lm(fit12.form, lm12)
+fit12.xy <- linmod(bool.data$x, bool.data$y)
+# hack: delete mismatching names so check.lm() doesn't fail
+names(lm12$coefficients) <- NULL     # were "(Intercept)" "xTRUE"
+names(fit12.xy$coefficients) <- NULL # were "(Intercept)" "V1"
+check.lm(fit12.xy, lm12)
 
 cat("==data.frame with strings\n")
 
@@ -729,40 +579,59 @@ colnames(df.with.string) <- c("num1", "num2", "string")
 
 fit30.form <- linmod(num1~num2, df.with.string)
 lm30       <- lm(num1~num2, df.with.string)
-check(fit30.form, lm30)
+check.lm(fit30.form, lm30)
 
 fit31.form <- linmod(num1~., df.with.string)
 lm31       <- lm(num1~., df.with.string)
-check(fit31.form, lm31)
+check.lm(fit31.form, lm31)
 
-expect.err(try(linmod(string~., df.with.string)), "y is not numeric or logical")
+expect.err(try(linmod(string~., df.with.string)), "'y' is not numeric or logical")
 
 vec <- c(1,2,3,4,3)
-expect.err(try(linmod(df.with.string, vec)), "non-numeric column in x")
-expect.err(try(linmod(tit$pclass, tit$survived)), "non-numeric column in x")
+expect.err(try(linmod(df.with.string, vec)), "non-numeric column in 'x'")
+expect.err(try(linmod(tit$pclass, tit$survived)), "non-numeric column in 'x'")
 
 cat("==x is singular\n")
 
 set.seed(1)
 x2 <- matrix(rnorm(6), nrow=2)
 y2 <- c(1,2)
-expect.err(try(linmod(y2~x2)), "x is singular")
+expect.err(try(linmod(y2~x2)), "'x' is singular")
 
 x3 <- matrix(1:10, ncol=2)
 y3 <- c(1,2,9,4,5)
-expect.err(try(linmod(y3~x3)), "x is singular")
+expect.err(try(linmod(y3~x3)), "'x' is singular")
+
+expect.err(try(linmod(trees[1,1:2], trees[1,3])), "'x' is singular (it has 3 columns but its rank is 1)")
+
+cat("==nrow(x) does not match length(y)\n")
+
+x4 <- matrix(1:10, ncol=2)
+y4 <- c(1,2,9,4)
+expect.err(try(linmod(x4, y4)), "nrow(x) is 5 but length(y) is 4")
+
+x5 <- matrix(1:10, ncol=2)
+y5 <- c(1,2,9,4,5,9)
+expect.err(try(linmod(x5, y5)), "nrow(x) is 5 but length(y) is 6")
+
+cat("==y has multiple columns\n")
+
+vec <- c(1,2,3,4,3)
+y2 <- cbind(c(1,2,3,4,9), vec^2)
+expect.err(try(linmod(vec, y2)), "nrow(x) is 5 but length(y) is 10")
+expect.err(try(linmod(y2~vec)), "nrow(x) is 5 but length(y) is 10")
 
 cat("==NA in x\n")
 
 x <- tr[,1:2]
 y <- tr[,3]
 x[2,2] <- NA
-expect.err(try(linmod(x, y)), "NA in x")
+expect.err(try(linmod(x, y)), "NA in 'x'")
 
 x <- tr[,1:2]
 y <- tr[,3]
 y[9] <- NA
-expect.err(try(linmod(x, y)), "NA in y")
+expect.err(try(linmod(x, y)), "NA in 'y'")
 
 cat("==misc tests with different kinds of data\n")
 
@@ -774,7 +643,7 @@ stopifnot(sum(abs(predict(a40, newdata=data3[1:3,]) - c(2.71429, 2.00000, 3.0000
 
 data4 <- data.frame(s=c("a", "b", "c", "a", "a"), num=c(1,9,4,2,6), y=c(1,2,3,5,3), stringsAsFactors=T)
 stopifnot(sapply(data4, class) == c("factor", "numeric", "numeric"))
-expect.err(try(linmod(data4[,1:2], data4[,3])), "non-numeric column in x")
+expect.err(try(linmod(data4[,1:2], data4[,3])), "non-numeric column in 'x'")
 
 # following gives no error (and matches lm)
 a41 <- linmod(y~., data=data4)
@@ -784,18 +653,103 @@ data5 <- data.frame(s=c("a", "b", "c", "a", "a"), num=c(1,9,4,2,6), y=c(1,2,3,5,
 stopifnot(sum(abs(predict(a41, newdata=data5[1:3,1:2]) - c(2.71429, 2.00000, 3.00000))) < 0.001)
 
 data6 <- data.frame(s=c("a", "b", "c", "a9", "a"), num=c(1,9,4,2,6), y=c(1,2,3,5,3), stringsAsFactors=T)
-expect.err(try(predict(a41, newdata=data6[1:3,1:2])), "ncol(x) is 5 but should be 3")
+expect.err(try(predict(a41, newdata=data6[1:3,1:2])), "ncol(newdata) is 4 but should be 3")
 
 expect.err(try(predict(a41, newdata=1:9)), "object 's' not found") # issued by model.matrix in predict.linmod
 
-cat("==multiple response model\n")
-# TODO multiple response models not fully tested
+expect.err(try(predict(a41, newdata=data.frame())), "'newdata' is empty")
 
-y.2 <- cbind(c(1,2,3,4,9), vec^2)
-fit100.mat <- linmod(vec, y.2)
-lm100 <- lm.fit(cbind(1, vec), y.2)
-stopifnot(coef(fit100.mat) == coef(lm100))
-# stopifnot(names(coef(fit100.mat)) == names(coef(lm100)))
+tr.na <- trees
+tr.na[9,3] <- NA
+expect.err(try(linmod(Volume~.,data=tr.na)), "NA in 'y'")
+expect.err(try(linmod(tr.na[,1:2], tr.na[,3])), "NA in 'y'")
+
+tr.na <- trees
+tr.na[10,1] <- NA
+expect.err(try(linmod(Volume~.,data=tr.na)), "NA in 'x'")
+expect.err(try(linmod(tr.na[,1:2], tr.na[,3])), "NA in 'x'")
+
+a42 <- linmod(trees[,1:2], trees[, 3])
+newdata1 <- data.frame(Girth=20)
+expect.err(try(predict(a42, newdata=newdata1)), "ncol(newdata) is 1 but should be 2")
+expect.err(try(predict(a42, newdata=data.frame())), "'newdata' is empty")
+newdata.with.NA <- data.frame(Girth=20, Height=NA)
+expect.err(try(predict(a42, newdata=newdata.with.NA)), "NA in 'newdata'")
+
+a43 <- linmod(Volume~.,data=trees)
+expect.err(try(predict(a43, newdata=newdata.with.NA)), "NA in 'newdata'")
+
+# TODO ncol(newdata) is 3 but should be 3
+# expect.err(try(predict(a41, newdata=data.frame(s=1, num=2, y=3))), "xxx")
+
+y6 <- 1:5
+x6 <- data.frame()
+expect.err(try(linmod(x6, y6)), "'x' is empty")
+
+y7 <- data.frame()
+x7 <- 1:5
+expect.err(try(linmod(x7, y7)), "'y' is empty")
+
+# duplicated column names
+data7 <- matrix(1:25, ncol=5)
+colnames(data7) <- c("y", "x1", "x1", "x3", "x4")
+expect.err(try(linmod(data7[,-1], data7[,1])), "column name \"x1\" in 'x' is duplicated")
+
+colnames(data7) <- c("y", "x1", "x2", "x2", "x4")
+expect.err(try(linmod(data7[,-1], data7[,1])), "column name \"x2\" in 'x' is duplicated")
+
+colnames(data7) <- c("y", "x1", "x2", "x2", "x2")
+expect.err(try(linmod(data7[,-1], data7[,1])), "column name \"x2\" in 'x' is duplicated")
+
+# column name V2 will be created but it clashes with the existing column name
+colnames(data7) <- c("y", "V2", "", "V3", "V4")
+expect.err(try(linmod(data7[,-1], data7[,1])), "column name \"V2\" in 'x' is duplicated")
+
+# missing column names
+trees1 <- trees
+colnames(trees1) <- NULL
+cat("a52\n")
+a52 <- linmod(trees1[,1:2], trees1[,3])
+print(summary(a52))
+
+trees1 <- trees
+colnames(trees1) <- c("", "Height", "Volume") # was Girth Height Volume
+cat("a53\n")
+a53 <- linmod(trees1[,1:2], trees1[,3])
+print(summary(a53))
+cat("a53.formula\n")
+expect.err(try(linmod(Volume~., data=trees1)), "attempt to use zero-length variable name")
+
+# very long names to test formatting in summary.linmod
+trees1 <- trees
+colnames(trees1) <- c("Girth.a.very.long.name.in.fact.an.exceptionally.long.name",
+                      "Height.a.very.long.name.in.fact.an.exceptionally.long.name",
+                      "Volume.a.very.long.name.in.fact.an.exceptionally.long.name")
+cat("a55\n")
+a55 <- linmod(Volume.a.very.long.name.in.fact.an.exceptionally.long.name~
+              Girth.a.very.long.name.in.fact.an.exceptionally.long.name+
+              Height.a.very.long.name.in.fact.an.exceptionally.long.name,
+              data=trees1)
+print(summary(a55))
+options(show.signif.stars=FALSE) # god i hate significance stars
+print(summary(a55))
+
+# intercept-only model
+a56.form <- linmod(Volume~1, data=trees)
+print(summary(a56.form))
+stopifnot(length(coef(a56.form)) == 1)
+plotres(a56.form)
+# TODO following gives error: gen.colnames(x, "x", "x", trace, xname = "x"): length(prefix) <= NCOL(x) is not TRUE
+# plotmo(a56.form)
+expect.err(try(linmod(rep(1, length.out=nrow(trees)), trees$Volume)), "'x' is singular (it has 2 columns but its rank is 1)")
+
+# various tests for bad args
+expect.err(try(linmod(trees[,1:2])), "no 'y' argument")
+
+expect.err(try(linmod(Volume~., data=trees, nonesuch=99)), "unused argument (nonesuch = 99)")
+expect.err(try(linmod(trees[,1:2], trees[,3], nonesuch=linmod)), "unused argument (nonesuch = function (...)")
+expect.err(try(summary(linmod(trees[,1:2], trees[,3]), nonesuch=linmod)), "unused argument (nonesuch = function (...)")
+expect.err(try(print(linmod(trees[,1:2], trees[,3]), nonesuch=linmod)), "unused argument (nonesuch = function (...)")
 
 if(!interactive()) {
     dev.off()        # finish postscript plot
